@@ -29,11 +29,12 @@
 # along with uicilibris.  If not, see <http://www.gnu.org/licenses/>.
 
 
-import sys, url, StringIO, re
+import sys, url, StringIO, re, os, urllib2
 from regularExpressions import *
 from BeautifulSoup import BeautifulSoup
 from plugin.plugin import plugins
 from templateParser import imageParser
+from PyQt4.QtCore import SIGNAL
 
 wParser=None # global variable: the last and only wikiParser object
              # this global is used by some plugins.
@@ -41,19 +42,33 @@ wParser=None # global variable: the last and only wikiParser object
 def getWikiContents(completeUrl):
     """
     @param completeUrl the url of a wiki page, encoded in utf-8
-    @return the wiki code for the page, else a void string
+    @return the base URL of the wiki and the wiki code for the page, else two void strings
     """
-    completeUrl=completeUrl.replace("index.php/","index.php?title=")+"&action=edit"
+    result=("","")
+    baseUrl=re.match(r"^(http://[^/]+/).*",completeUrl).group(1)
+    try:
+        sPage=url.urlopen(completeUrl)
+    except urllib2.HTTPError:
+        return result
+    if sPage:
+        soup=BeautifulSoup(sPage.read())
+        pattern=re.compile('/(.*)/index.php\?title=.*action=edit$')
+        editAddress=soup.find(href=pattern)
+        if editAddress != None:
+            localUrl=editAddress.attrMap["href"]
+            base=pattern.match(localUrl).group(1)
+            completeUrl=baseUrl+localUrl
+        else:
+            return result
+    else:
+        return result
     bPage=url.urlopen(completeUrl)
     if bPage:
         soup=BeautifulSoup(bPage.read())
         area=soup.find('textarea', id="wpTextbox1")
         if area and len(area.contents)>0 :
-            return area.contents[0]
-        else:
-            return ""
-    else:
-        return ""
+            result = (baseUrl+base, area.contents[0])
+    return result
 
 def findDivById(n,_id):
     """
@@ -117,13 +132,15 @@ class lineJoiner:
     a class to join lines taking in account some wiki syntax
     """
 
-    def __init__(self, f, encoding=None):
+    def __init__(self, f, encoding=None, oldWiki2=False):
         """
         the constructor
         @ param f a file-like readable object
         @param encoding triggers some specific encoding if different from None
+        @param oldWiki2 True to process an old style wiki2beamer source
         """
         self.encoding=encoding
+        self.oldWiki2=oldWiki2
         self.lines=self.joinLines(f.readlines())
 
     def getLines(self):
@@ -137,7 +154,6 @@ class lineJoiner:
         join lines ending with unescaped percent signs,
         unless inside codemode or nowiki mode
         @param lines a list of text lines
-        @return the lines proporly joined
         """
         nowikimode = False
         codemode = False
@@ -155,7 +171,12 @@ class lineJoiner:
             else:
                 l = _l
 
-            if l=="" and r and r[-1]!="\n\n" :
+            #restore markups like <math></math>, etc.
+            for m in ('math', 'code', 'ref'):
+                l=re.sub('&amp;lt;(/?)'+m+'&amp;gt;', r'<\1'+m+'>', l)
+            l=re.sub('"&amp;lt;references */&amp;gt;','<references />',l)
+
+            if (not self.oldWiki2) and l=="" and r and r[-1]!="\n\n" :
                 l="\n\n" # simplify and keep the return codes for LaTeX
             if not (nowikimode or codemode) and (len(l) > 1) and (l[-1] == "%") and (l[-2] != "\\"):
                 s = s + l[:-1]
@@ -164,9 +185,12 @@ class lineJoiner:
             else: # when l=="" or codemode, or nowikimode, or ...
                 # output a line
                 s = s + l
-                if len(s)>0:
+                if (not self.oldWiki2) and len(s)>0:
                     r.append(s+"\n")
-                    s = ''
+                else:
+                    r.append(s)
+                s = ''
+                
 
         return r
     
@@ -174,11 +198,13 @@ class txtFileCacher:
     """
     a cache for text files
     """
-    def __init__(self):
+    def __init__(self, oldWiki2=False):
         """
         the constructor
+        @param oldWiki2 True to process an old style wiki2beamer source
         """
         self.reset()
+        self.oldWiki2=oldWiki2
 
     def __str__(self):
         return "%s" %self._cache
@@ -221,10 +247,10 @@ class txtFileCacher:
         """
         if not fileObj:
             f = open(filename, "r")
-            lines=lineJoiner(f, encoding).getLines()
+            lines=lineJoiner(f, encoding, oldWiki2=self.oldWiki2).getLines()
             f.close()
         else:
-            lines=lineJoiner(fileObj, encoding).getLines()
+            lines=lineJoiner(fileObj, encoding, oldWiki2=self.oldWiki2).getLines()
             self.addLines(filename, lines)
 
         return lines
@@ -241,7 +267,7 @@ class wikiParser:
     like LaTeX/Beamer
     """
 
-    def __init__(self, args, isatty=True, isUrl=False, report=False):
+    def __init__(self, args=[], isatty=True, isUrl=False, report=False, parent=None, latexReadyParam="", oldWiki2beamer=False):
         """
         The constructor
         @param args a list of filenames, or a single URL.
@@ -252,14 +278,30 @@ class wikiParser:
         @param isUrl is True to force the initialization, by considering arg as a single url which is supposed to be a normal wiki page
         @param report if True, messages are emitted to sys.stderr;
         if it is callable, it is invoked with the same messages
+        @param parent pointer to the main window
+        @param latexReadyParam the parameter for a signal to be emitted
+        upon completion of the process
+        @param oldWiki2beamer when True, uses the modes of wiki2beamer
         """
         global wParser
         wParser=self
         self.lines=[]
         input_files = []
-        self.cache=txtFileCacher()
         self.imageSet=set()
         self.report=report
+        self.parent=parent
+        if parent:
+            # creates a fake page name to initialize the parser by
+            # getting a first response from the mediawiki
+            parent.wb=self
+            self.baseAddress=parent.wikiIndex.replace("http://","")
+            self.host=re.match(r"([^/]+).*", self.baseAddress).group(1)
+            args=[parent.wikiIndex.encode("utf-8")+"/fakePage"]
+            isUrl=True
+            oldWiki2beamer=parent.autoOldStyle()
+        self.cache=txtFileCacher(oldWiki2= oldWiki2beamer)
+        self.latexReadyParam=latexReadyParam
+        self.oldWiki2beamer=oldWiki2beamer
         if not isatty:
             input_files.append('stdin')
             self.cache.read_file_to_lines(filename='stdin', fileObj=sys.stdin)
@@ -276,15 +318,32 @@ class wikiParser:
         for file_ in input_files:
             self.lines += self.include_file_recursive(file_)
 
-    def reloadCache(self, text, info=""):
+    def parseWikiSource(self):
+        """
+        parses a text provided by self.parent in its drop area
+        """
+        if not self.parent:
+            return
+        self.reloadCache(self.parent.wikiSource, oldWiki2beamer=self.oldWiki2beamer)
+        self.parent.emit(SIGNAL("latexready"), self.latexReadyParam)
+
+    def reloadCache(self, text, info="", oldWiki2beamer=False):
         """
         Reloads the cache from a given text, after running template processors.
         @param text a unicode string with wiki code.
         @param info some informative message
+        param oldWiki2beamer if True, we are in the old wiki2beamer mode
+        the text will be processed to revert some effects of
+        python-BeautifulSoup. Plugins will be applied, but no submission
+        will be done to the special page ExpandTemplates.
         """
         if info: print >> sys.stderr, "'%s'" %info
-        text=self.applyPLugins(text)
-        text=self.wikiTemplates(text)
+        if oldWiki2beamer:
+            text=text.replace("&lt;", "<")
+            text=text.replace("&amp;", "&")
+        text=self.applyPlugins(text)
+        if not oldWiki2beamer:
+            text=self.wikiTemplates(text)
         self.loadUrlLines(text+"\n")
         self.cache.reset()
         self.lines=self.cache.read_file_to_lines(filename='url', fileObj=self.urlLines, encoding="utf-8")
@@ -309,6 +368,32 @@ class wikiParser:
         """
         self.lines=self.cache.read_file_to_lines(filename='url', fileObj=self.urlLines)
         
+    def getImages(self, path, report=None):
+        """
+        gets the necessary images from the mediawiki
+        @param path the path to write images
+        @param report if True, this method will output a few messages on sys.stderr;
+        if report is a callback function, this function will be called with one parameter.
+        """
+        href=""
+        for img in self.imageSet:
+            completeUrl="http://%s/index.php/File:%s" %(self.baseAddress, img)
+            page=url.urlopen(completeUrl)
+            soup=BeautifulSoup(page.read())
+            divs=soup.findAll("div", id="file")
+            for div in divs:
+                a=div.find("a")
+                href=a["href"]
+                if report==True:
+                    print >> sys.stderr, "'%s'" %href
+                elif callable(report):
+                    report("'%s'" %href)
+            imgData=url.urlopen("http://%s/%s" %(self.host,href))
+            imgFile=open(os.path.join(path,img),"w")
+            imgFile.write(imgData.read())
+            imgFile.close()
+        return
+    
     def registerImage(self, img):
         """
         registers an image filename to retreive it later
@@ -329,33 +414,37 @@ class wikiParser:
         """
         if len(arg)<1:
             return False
-        pattern=re.compile("http://(.+)/index.php/(.+)")
+        pattern=re.compile("http://(.+)/[^/]+/(.+)")
         m=pattern.match(arg[0])
         if m:
             self.baseAddress=m.group(1).decode("utf-8")
             self.host=re.findall("([^/]+).*", self.baseAddress)[0]
             basePage=m.group(2).decode("utf-8")
-            text=self.getWikiContents(basePage)
+            text=self.getWikiContentsByTitle(basePage)
             if text:
                 wikiAddresses=re.findall("\[\[([^\]]+)\]\]", text)
                 self.include_addresses(wikiAddresses, cbInfo=self.toStdErr)
                 return True
         return False
 
-    def getWikiContents(self, title):
+    def getWikiContentsByTitle(self, title):
         """
+        Gets the wiki code from the website, given a page's title
         @param title the title of a wiki page
         @return the wiki code for the page, else a void string
         """
         completeUrl="http://%s/index.php?title=%s" %(self.baseAddress,url.quote_plus(title.encode("utf-8")))
-        return getWikiContents(completeUrl)
+        return getWikiContents(completeUrl)[1]
 
-    def applyPLugins(self, s):
+    def applyPlugins(self, s):
         """
         pre-processes a few simple templates which have a precise definition for
         LaTeX, then processes the images.
         @param s the string to be processed
         """
+        #== recodes File:.*.jpg, Fichier:.*.jpg, etc. into Image: templates
+        s=re.sub(r"\[\[(File|Fichier):([-_0-9a-zA-Z]*)\.(jpg|JPG|png|PNG|gif|GIF)",r"[[Image:\2.\3",s)
+        #print "GRRR", s
         #== the parsers of every plugin are applied, then the image parser
         parsers=map(lambda c: c(), plugins)+[imageParser(imgFun)]
         for parser in parsers:
@@ -396,14 +485,14 @@ class wikiParser:
             self.baseAddress=m.group(1).decode("utf-8")
             self.host=re.findall("([^/]+).*", self.baseAddress)[0]
             basePage=m.group(2).decode("utf-8")
-            text=self.getWikiContents(basePage)
+            text=self.getWikiContentsByTitle(basePage)
             if text==None:
                 text="Error: the page %s does not exist" %completeUrl
             if self.report and completeUrl:
                 print >> sys.stderr, "'%s'" %completeUrl
             elif callable(self.report) and completeUrl:
                 self.report("'%s'" %completeUrl)
-            text=self.applyPLugins(text) 
+            text=self.applyPlugins(text) 
             text=self.wikiTemplates(text)
             text=text.encode("utf-8")
         self.loadUrlLines(text)
@@ -432,13 +521,17 @@ class wikiParser:
         """
         text=""
         for a in wikiAddresses:
+            # prepares the wikiAddress
+            pos=a.find("|")
+            if pos > 0 and pos < len(a):
+                a=a[:pos]
             if cbInfo: cbInfo(a)
-            contents=self.getWikiContents(a)
-            contents=self.applyPLugins(contents) 
+            contents=self.getWikiContentsByTitle(a)
+            contents=self.applyPlugins(contents) 
             processedContents=self.wikiTemplates(contents)
-            text+="<!-- uicilibris: begin '%s' -->\n" %a
+            text+="\n<!-- uicilibris: begin '%s' -->\n" %a
             text+=processedContents+"\n"
-            text+="<!-- uicilibris: end '%s' -->\n" %a
+            text+="\n<!-- uicilibris: end '%s' -->\n" %a
         # enforce the same encoding as ordinary text files
         text=text.encode("utf-8")
         # and make it like a text file
